pro make_image_tiles,modis_name
;---------------------------------------------------------------------
; The argument to this program is the name of a new MODIS tile image
; (jpeg), the output is a set of super-overlay imagery and
; cooresponding kml files for Google Earth. This assumes that
; the input jpeg is a higher resolution than 256 x 256.  The new super
; overlay images and kml files will also be copied to the Terra or Aqua 
; "current" folder for updating of the global data.
;
; INPUT: modis_name - The name of the new JPG image to be operated on
;
; HARD WIRED: A few paths are hard wired
;  location of the modis input images: '../tiles/'
;  location of the super overlay images and kml files (e.g., for image
;    tile197.terra.2008210.jpg): '../kml/2008210/tile197.terra/'
;  The jpeg quality is set to 75 (out of 100)
;
; NEEDED: gridindex_to_latlon.pro, modis_kml.pro
;
; AUTHOR: Amato Evan - CIMSS/UW-Madison amatoe@ssec.wisc.edu
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; Define paths to the MODIS image and the new super overlay tiles
;---------------------------------------------------------------------
sat_name = strmid(modis_name,8,4) ;terr or aqua
  if(sat_name eq 'terr')then sat_name = 'terra'
date = strmid(modis_name,strpos(modis_name,'.jpg')-7,7)

modis_path = '../tiles/tile'+strmid(modis_name,4,3)+'/'

kml_path = '../kml/'+date+'/tile'+strmid(modis_name,4,3)+'_'+sat_name+'/'

tile_index = 1*strmid(modis_name,4,3)

;---------------------------------------------------------------------
;Read in MODIS imagery
;---------------------------------------------------------------------
read_jpeg,modis_path+modis_name,z
n = size(z)
if(n[0] eq 3)then begin
imx = n[2]
imy = n[3]

red = reform(z[0,*,*])
green = reform(z[1,*,*])
blue = temporary(z)
blue = reform(blue[2,*,*])

;---------------------------------------------------------------------
;Make lat/lon arrays - images coords. defined with gridindex_to_latlon
;---------------------------------------------------------------------
gridindex_to_latlon, tile_index, 10, lat, lon
limits = [lat,lon,lat+10.,lon+10.]

dlon = (abs(limits[3]-limits[1]))/double(imx-1)
dlat = (abs(limits[2]-limits[0]))/double(imy-1)

lon = limits[1] + dindgen(imx)*dlon
lat = limits[0] + dindgen(imy)*dlat

;---------------------------------------------------------------------
; Make a template for all the images that will be made
;---------------------------------------------------------------------
;number of levels -- incldg. 0th level
n_le = ceil(max([alog(imx/256.)/alog(2.), $
                      alog(imy/256.)/alog(2.)]))

;count the number of images & files to make
count = 0
n = 2
while(n gt 1)do begin
    if(count eq 0)then begin
       x = ceil(imx/256.)
       y = ceil(imy/256.)
       n = x * y
    endif else begin
       x = ceil(x/2.)
       y = ceil(y/2.)
       n = x * y
    endelse
    count = count + n
endwhile
template = dblarr(count,5) - 999;[index,[res. level,coords.]]

;---------------------------------------------------------------------
; Get index number, level, and coords. for each image.  The while loop
; starts at the highest resolution and gets more coarse until n = 1,
; signifying that the resolution only permits 1-image to be made.  Do
; this first to get coords. for every image before trying to make the
; kml files' links.
;---------------------------------------------------------------------
count = 0
i_level = 0
n = 2
while(n gt 1)do begin
    if(count eq 0)then begin
        x = ceil(imx/256.)
        y = ceil(imy/256.)
        n = x * y
        for i = 0, x-1 do for j = 0, y -1 do begin
            template[count,0] = n_le - i_level

            template[count,1] = lat[j*255]
            template[count,2] = lon[i*255]

            template[count,3] = lat[min([j*255+255,imy-1])]
            template[count,4] = lon[min([i*255+255,imx-1])]

            count = count + 1
        endfor
        i_level = i_level + 1
    endif else begin
       x = ceil(x/2.)
       y = ceil(y/2.)
       n = x * y
       for i = 0, x-1 do for j = 0, y -1 do begin
            template[count,0] = n_le - i_level

            template[count,1] = lat[j*255*2^(i_level)]
            template[count,2] = lon[i*255*2^(i_level)]

            bound_j = min([imy-1,(j*255+255)*2^(i_level)])
            bound_i = min([imx-1,(i*255+255)*2^(i_level)])
            template[count,3] = lat[bound_j]
            template[count,4] = lon[bound_i]

            count = count + 1
        endfor
        i_level = i_level + 1
    endelse
endwhile
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; loop through resolutions and now create images/kml files
;---------------------------------------------------------------------
for n = 0, count - 1 do begin
    res = template[n,0]

    ;get bounds for image
    ul_j = where(lat eq template[n,3])
    ul_i = where(lon eq template[n,2])

    ll_j = where(lat eq template[n,1])
    ll_i = ul_i

    ur_j = ul_j
    ur_i = where(lon eq template[n,4])

    lr_j = ll_j
    lr_i = ur_i

    di = lr_i-ll_i+1
    dj = ul_j-ll_j+1

    ll_i = ll_i[0]
    lr_i = lr_i[0]
    ll_j = ll_j[0]
    ul_j = ul_j[0]
    
    ;make the image
    if(res eq n_le)then begin
        ix = min([di,256])
        iy = min([dj,256])        
        tred   = red[ll_i:lr_i,ll_j:ul_j]
        tgreen = green[ll_i:lr_i,ll_j:ul_j]
        tblue  = blue[ll_i:lr_i,ll_j:ul_j]
    endif else begin
        ; Experimental - to fix some resolution issues (8/6)
        factor = max([round(dj/256),round(di/256.)])
        ix = di/factor
        iy = dj/factor
	ix = ix[0]
	iy = iy[0]
        tred   = regrid(red[ll_i:lr_i,ll_j:ul_j],ix,iy)
        tgreen = regrid(green[ll_i:lr_i,ll_j:ul_j],ix,iy)
        tblue  = regrid(blue[ll_i:lr_i,ll_j:ul_j],ix,iy)    
    endelse

    filename = strcompress(n,/remove_all)+'.jpg'

    image = bytarr(3,ix,iy)
    image[0,*,*] = tred
    image[1,*,*] = tgreen
    image[2,*,*] = tblue

    if(min(image) eq 255) then begin
       template[n,*] = -999
    endif else begin
       spawn, 'mkdir -p '+kml_path
       ;write the jpeg file
       write_jpeg,kml_path+filename,image,true=1,quality=75
       ;write the kml file
       modis_kml,kml_path,template,reform(template[n,*]),n
    endelse

endfor
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; Copy the new superoverlay images and .kml files over to the current
; directory for this satellite
;---------------------------------------------------------------------
print, '*****'
print, 'COPY TO CURRENT DIR IS DISABLED'
;from = kml_path
;to = '../kml/current/tile'+strmid(modis_name,4,3)+'_'+sat_name+'/'
;file_copy,from,to,/overwrite,/recursive

;---------------------------------------------------------------------
endif

end
